#!/bin/bash

###############################################################################
##   check.sh: system check.                                                 ##
##                                                                           ##
##   Copyright (C) 2007 by Colomban Wendling                                 ##
##   ban-ubuntu@club-internet.fr                                             ##
##                                                                           ##
##   This program is free software; you can redistribute it and/or modify    ##
##   it under the terms of the GNU General Public License as published by    ##
##   the Free Software Foundation; either version 2 of the License, or       ##
##   (at your option) any later version.                                     ##
##                                                                           ##
##   This program is distributed in the hope that it will be useful,         ##
##   but WITHOUT ANY WARRANTY; without even the implied warranty of          ##
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           ##
##   GNU General Public License for more details.                            ##
##                                                                           ##
##   You should have received a copy of the GNU General Public License       ##
##   along with this program; if not, write to the                           ##
##   Free Software Foundation, Inc.,                                         ##
##   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.               ##
###############################################################################

## this script check dependencies for a program
## modify variables variables to adapt to your program







###############################################################################
##   start of variable declaration                                           ##
###############################################################################

###############################################################################
##   variables variables (programmer should modify it)                       ##

# name of application we are checking dependecies
PACKAGE="SokoBan2"

## NOTE
## all lists are checked from left to right

# list of directories where search H files
H_DIRS="/usr/include /usr/local/include /usr/local/lib*/gcc/*/*/include /usr/lib*/gcc/*/*/include"

H_GEN="stdio.h stdlib.h string.h stdarg.h ctype.h"
H_LOCALE="libintl.h locale.h"
H_SDL="SDL/SDL.h SDL/SDL_ttf.h SDL/SDL_image.h"
# list of H files
H="$H_GEN $H_SDL $H_LOCALE"


# list of directories chere search libraries
L_DIRS="/usr/local/lib* /usr/lib* /lib*"

L_SDL="SDL SDL_ttf SDL_image"
# list of libraries (.a or .lib files)
L="$L_SDL"


# list of required applications
A="gcc make sdl-config msgfmt"






###############################################################################
##   static variables (script variables DO NOT MODIFY)                       ##

verbose=

err=0


###############################################################################
##   start of functions declaration                                          ##
###############################################################################

# print an error
function ee()
{
   local ecode=1 msg
   
   for i; do
      case $i in
         -e=*)
            ecode=${i:3}
            ;;
         *)
            msg="$msg $i"
            ;;
      esac
   done
   
   
   echo -e "EE: $msg" >&2
   exit $ecode
}
# print warning
function ww()
{
   echo -e "WW: $@" >&2
}
# print info
function ii()
{
   echo -e "II: $@" >&1
}



# check if a program is installed
function require()
{
   local cmd
   
   [ -z "$1" ] && return
   
   cmd=$(which "$1")
   
   if [ -z "$cmd" ];then
      ee -e=127 "required command '$1' not found!"
   else
      ii "'$1' found at '$cmd'"
   fi
}


# print $1 if $verbose is set as true or $2 else
function verbose()
{
   local eopts mtrue mfalse
   
   for i; do
      case $i in
         -e)
            eopts="$eopts -e"
            ;;
         -n)
            eopts="$eopts -n"
            ;;
         *)
            if   [ -z "$mtrue" ];then
               mtrue="$i"
            elif [ -z "$mfalse" ];then
               mfalse="$i"
            else
               ww "Unused parameter '$i'"
            fi
            ;;
      esac
   done
   
   
   if [ $verbose ];then
      echo "$mtrue"
   else
      echo "$mfalse"
   fi
}


#print help and exit
function help()
{
   cat << EOF

Usage: ${0##*/} [-h|--help] [-v] [--h-dir+=DIR] [--lib-dir+=DIR]

Options:

   -h | --help                print this help ant exit
   -v | --verbose             be verbose
   --add-header-dir=DIR | --h-dir+=DIR
                              add DIR to the header search path
   --add-library-dir=DIR | --lib-dir+=DIR
                              add DIR to the library search path

EOF
   
   exit 0
}

###############################################################################
##   start of main program                                                   ##
###############################################################################

# Checking command-line arguments
for i; do
   case $i in
      # verbose
      -v|--verbose)
         verbose=1
         ;;
      # add a lib dir
      --add-library-dir=*|--add-lib-dir=*|--library-dir+=*|--lib-dir+=*)
         L_DIRS="${i#*=} $L_DIRS"
         ii "adding '${i#*=}' to libraries search directories"
         ;;
      # add header dir
      --add-header-dir=*|--add-h-dir=*|--header-dir+=*|--h-dir+=*)
         H_DIRS="${i#*=} $H_DIRS"
         ii "adding '${i#*=}' to headers search directories"
         ;;
      #help
      -h|--help)
         help
         ;;
      # others
      *)
         echo "Unrecognized option '$i'" >&2
         help
   esac
done


# start infos, welcome to user ^^
echo 
echo -e "\tPlease wait a few seconds while checking for compile requirements of $PACKAGE..."


# Check for H files
echo -e "\n\tChecking for headers files..."
for h in $H; do
   echo -n "Checking for $h... "
   
   OK= # inintialization of OK var
   
   for d in $H_DIRS; do
      if [ -f "$d/$h" ];then
         OK=1
         break
      fi
   done
   
   if ! [ -z $OK ];then
      echo -ne "\E[032mOK\E[0m."
      
      verbose " (found at $d/$h)"
   else
      echo -e "\E[031mfailed\E[0m!" >&2
      
      echo
      echo -e "\tRequire header file '$h' not found!"
      echo -e "\tPlease check you have this file and if I cant find it."
      echo
      
      #exit 1
      err=1
   fi
done


# Check for libs
echo -e "\n\tChecking for libraries..."
for l in $L; do
   echo -n "Checking for $l... "
   
   OK= # inintialization of OK var
   
   for d in $L_DIRS; do
      if [ -f "$d/lib$l.a" ];then
         lf="$d/lib$l.a"
         OK=1
         break
      elif [ -f "$d/lib$l.la" ];then
         lf="$d/lib$l.la"
         OK=1
         break
      elif [ -f "$d/lib$l.so" ];then
         lf="$d/lib$l.so"
         OK=1
         break
      fi
   done
   
   if ! [ -z $OK ];then
      echo -ne "\E[032mOK\E[0m."
      
      verbose " (found at $d/$l)"
   else
      echo -e "\E[031mfailed\E[0m!" >&2
      
      echo
      echo -e "\tRequire '$l' library not found!"
      echo -e "\tPlease check you have this library and if I cant find it."
      echo
      
      #exit 1
      err=1
   fi
done


# check for require application
echo -e "\n\tChecking for utilities..."
for a in $A; do
   echo -n "Checking for $a... "
   
   cmd=$(which "$a" 2>/dev/null)
   
   if [ -x "$cmd" ]; then
      echo -ne "\E[032mOK\E[0m."
      
      verbose " (found at $cmd)"
   else
      echo -e "\E[031mfailed\E[0m!" >&2
      
      echo
      echo -e "\tRequire '$a' application not found!"
      echo -e "\tPlease check you have this application insttaled and if I cant find it."
      echo
      
      #exit 1
      err=1
   fi
done


echo

if [ $err = 0 ]; then
   echo
   echo -e "\tGreat!"
   echo "You can now type 'make' to compile $PACKAGE!"
   echo
else
   echo
   echo -e "\t\E[031mError\E[0m!"
   echo "Some required files/applications was not found on your system."
   echo "You have to fix errors reported below before compiling $PACKAGE."
   echo
   
   exit 1
fi

exit 0
