/*
 *  SokoBan, a Sokoban game.
 *  Copyright (C) 2008  Colomban Wendling <ban@herbesfolles.org>
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include <string.h>
#include <stdio.h>

#include "sokoban.h"
#include "level.h"
#include "i18n.h"
#include "set.h"


/**
 * \defgroup set Multilevel sets functions
 * Multilevel sets loading, unloading & managment
 */

/** @{ */


/**
 * \brief current set string
 */
static char* _set = NULL;


/**
 * \brief get filename of level at ID \p id
 * \param id the ID (starting at 0) of the level to load
 * \return the level path on success or NULL if ID does'nt exist or on memory
 *         allocation error.
 */
char* set_getNthLevel (const unsigned int id)
{
   int i, j;
   unsigned int n = 0;
   char* level;
   
   if (_set == NULL)
      return NULL;
   
   /* allocation of level string */
   if (!(level = malloc (strlen (_set) +1)))
   {
      memalloc_errormsg ();
      return NULL;
   }
   
   
   for (i=0, j=0; _set[i] != '\0'; i++)
   {
      if (_set[i] == '\n')
      {
         level[j] = '\0';
         
         if (n == id)
         {
            #ifdef DEBUG
            printf ("level: %s\n", level);
            #endif
            
            return level;
         }
         
         n++;
         j = 0;
      }
      else
      {
         level[j] = _set[i];
         j++;
      }
   }
   
   free (level);
   return NULL;
}

/**
 * \def set_getFirstLevel
 * \brief get filename of the first level of the current set
 * \return the level path on success or NULL on memory allocation error.
 * 
 * \see set_getNthLevel
 */

/**
 * FIXME: make this function use set_getNthLevel()
 * \brief get filename of the last level of the current set
 * \return the level path on success or NULL on memory allocation error.
 */
char* set_getLastLevel (void)
{
   int i, j;
   char* level;
   
   if (_set == NULL)
      return NULL;
   
   /* allocation of level string */
   if (!(level = malloc (strlen (_set) +1)))
   {
      memalloc_errormsg ();
      return NULL;
   }
   
   
   for (i=0, j=0; _set[i] != '\0'; i++)
   {
      if (_set[i] == '\n')
      {
         level[j] = '\0';
         
         j = 0;
      }
      else
      {
         level[j] = _set[i];
         j++;
      }
   }
   
   return level;
}


char* set_getLevelFromNameWithExt (const char* name)
{
   int i;
   char* level;
   
   for (i = 0; (level = set_getNthLevel (i)) != NULL; i++)
   {
      if (strcmp (level, name) == 0)
         break;
      else
      {
         free (level);
         level = NULL;
      }
   }
   
   return level;
}

char* set_getLevelFromName (const char* name)
{
   char* level;
   char* fullname;
   char* ext = "."LEVEL_EXT;
   
   fullname = malloc (strlen (name) + strlen (ext) + 1);
   if (!fullname) return NULL;
   
   fullname = strcpy (fullname, name);
   fullname = strcat (fullname, ext);
   
   /*if (sprintf (fullname, "%s%s", name, ext) < 0)
   {
      free (fullname);
      return NULL;
   }*/
   
   level = set_getLevelFromNameWithExt (fullname);
   
   free (fullname);
   
   return level;
}


/**
 * \brief load a set from a file steram
 * \param fp the file stream from where get data
 * \return 0 on success, -1 on error.
 * 
 * Load a set from a file steram.
 * Note that the loaded set should be unloaded with set_close() after use.
 */
int set_openFromStream (FILE *fp)
{
   char c;
   int i;
   
   if (_set != NULL)
   {
      fprintf (stderr, _("** Internal error: _set not closed.\n"));
      return -1;
   }
   
   if (!(_set = malloc (SET_MAXLEN)))
   {
      memalloc_errormsg ();
      return -1;
   }
   
   
   for (i=0; i < SET_MAXLEN; i++)
   {
      switch ((c = fgetc (fp)))
      {
         case EOF:
            _set[i] = '\0';
            i = SET_MAXLEN; /* fake pour quitter la boucle :-° */
            break;
         
         case '\r':
            i --;
            break;
         
         default:
            _set[i] = c;
      }
   }
   
   #ifdef DEBUG
   printf ("%s\n", _set);
   #endif
      
   return 0;
}

/**
 * \brief load a set from a file
 * \param filename the path of the set to load
 * \return -1 on error, the return value of set_openFromStream() else.
 * \see set_openFromStream
 * 
 * Load a set from a file using set_openFromStream().
 * Note that the loaded set should be unloaded with set_close() after use.
 */
int set_open (const char* filename)
{
   FILE *fp;
   int rv = -1;
   
   fp = fopen (filename, "r");
   
   if (fp != NULL)
   {
      rv = set_openFromStream (fp);
      fclose (fp);
   }
   else
      perror (_("** Failed to open set"));
   
   return rv;
}

/**
 * \brief close the currently opened set
 */
void set_close (void)
{
   free (_set), _set = NULL;
}


MyBool set_write (const char* filename)
{
   FILE *fp;
   MyBool rv = FALSE;
   
   fp = fopen (filename, "w");
   
   if (fp != NULL)
   {
      rv = set_writeToStream (fp);
      fclose (fp);
   }
   else
      perror (_("** Failed to write set:"));
   
   return rv;
}

MyBool set_writeToStream (FILE *stream)
{
   int i;
   char c;
   
   if (!_set)
      return FALSE;
   
   for (i = 0; (c = _set[i]) != '\0'; i++)
   {
      #ifdef DEBUG
      printf ("%c", c);
      #endif
      
      if (c == '\n')
      {
         if (fputs (EOL, stream) == EOF)
         return FALSE;
      }
      else
      {
         if (fputc (c, stream) == EOF)
         return FALSE;
      }
   }
   
   return TRUE;
}


MyBool set_appendLevel (const char *level)
{
   /* check for set & level */
   if (!_set || !level) return FALSE;
   
   /* check if there is place to put new level */
   if (strlen (level) >= (SET_MAXLEN - strlen (_set))) return FALSE;
   
   strcat (_set, level);
   strcat (_set, "\n"); // level separator
   
   return TRUE;
}


/**
 * \brief load a level from the current set to the map
 * \param id the level ID of the set
 * \return 0 if no error, -1 if level \p id is not found in the set, -2 on
 *         malloc() error, -3 on level_load() error
 * \see set_getNthLevel
 * \see level_load
 */
int set_loadLevel (const unsigned int id)
{
   char* level;
   char* filename;
   int rv = 0;
   
   level = set_getNthLevel (id);
   
   if (!level)
      return -1;
   
   if (!(filename = malloc (strlen (DATADIR) + strlen (level) + 1)))
   {
      memalloc_errormsg ();
      
      free (level);
      return -2;
   }
   
   sprintf (filename, "%s%s", DATADIR, level);
   
   free (level);
   
   if (!level_load (filename))
      rv = -3;
   
   free (filename);
   
   return rv;
}

int set_loadLastLevel (void)
{
   char* level;
   char* filename;
   int rv = 0;
   
   level = set_getLastLevel ();
   
   if (!level)
      return -1;
   
   if (!(filename = malloc (strlen (DATADIR) + strlen (level) + 1)))
   {
      memalloc_errormsg ();
      
      free (level);
      return -2;
   }
   
   sprintf (filename, "%s%s", DATADIR, level);
   
   free (level);
   
   if (!level_load (filename))
      rv = -3;
   
   free (filename);
   
   return rv;
}


MyBool set_levelExists (const char *levelname)
{
   MyBool rv = FALSE;
   char *level;
   
   level = set_getLevelFromNameWithExt (levelname);
   
   if (level)
      rv = TRUE;
   
   free (level);
   
   return rv;
}


/** @} */
